# -*- coding: utf-8 -*-
# MeshDVS_TOCHIGI.py
#
# 　ノンパラメトリックDVR法に基づき、指定した日に移植した水稲の出穂日を予測し、栃木県における
# 分布図を描画するプログラム。
#
# ノンパラメトリックDVRの係数は、関数DVR100の辞書LUTに定義される。
# メイン関数は、日平均気温をDVR100に渡してDVRを得、それを積算して100となった日を記録する。
# この計算を、栃木県下の気象データが存在するメッシュ全てに繰り返し実施し、出穂日の分布を得る。
#
# 　このプログラムは、分布を画面に表示するだけでなく、png形式、CSV形式のファイルでも出力する。
#
# IPythonからの実行は次のようにします。
#
#	In [1]: run MeshDVS_TOCHIGI.py　<エンター>
#
#
#	OHNO, Hiroyuki 2014.03.10
#	HOSHI, kazyoshi 2015.12.2改
# =========================================================================
import numpy as np
import sys
argvs = sys.argv  # コマンドライン引数を格納したリストの取得
argc = len(argvs) # 引数の個数
from matplotlib.dates import DateFormatter,DayLocator
import matplotlib.pylab as plt
import matplotlib.colors as clr
import AMD_Tools3 as AMD

# 発育速度関数------------------------------------------
def DVR100(Ta):
#   気温クラス毎のDVRの定義
    LUT = {  0:0.0,
            10:0.2210,
            11:0.3973,
            12:0.5571,
            13:0.6949,
            14:0.8048,
            15:0.8832,
            16:0.9285,
            17:0.9618,
            18:0.9985,
            19:1.0550,
            20:1.1300,
            21:1.2091,
            22:1.2771,
            23:1.3280,
            24:1.3652,
            25:1.3903,
            26:1.4172,
            27:1.4422,
            28:1.4663 }
            
    Tclass = int(round(Ta)) #平均気温を四捨五入して整数化
    if Tclass < 10 :  #平均気温が10℃より低いときは
        Tclass = 0    #0℃として扱う
    if Tclass > 28 :  #平均気温が28℃以上の時は 
        Tclass = 28   #28℃として扱う
    DVR100 = LUT[ Tclass ]
    return DVR100

# プログラムのメイン関数------------------------------------------
if __name__ == "__main__":

     #計算年と移植日の設定
#    if (argc != 2):   # 引数が足りない場合は、その旨を表示
#        print ('使用方法：: 「run %s yyyy-mm-dd(移植年月日)」　と入力し、エンターキーを押します。' % argvs[0])
#        sys.exit()         # プログラムの終了
    argvs = ["",""]
    argvs[1] = "2017-05-04"
    year = argvs[1][0:4]    #引数の最初の4文字を取り出す
    Day_of_Transplanting = argvs[1][5:]    #引数の最初の6文字以降を取り出す

    #メッシュデータの置き場の設定
#    url = 'http://mesh.dc.affrc.go.jp/opendap/'    #データをメッシュ農業気象データ配信サーバーから取得する場合
    url = 'I:/AMD'  #データを外付けUSBから取得する場合(ドライブレターは適宜変更して下さい)
    url = 'http://mesh.dc.affrc.go.jp/opendap/'
    #メッシュデータの取得
    timedomain = [ year+'-'+Day_of_Transplanting, year+'-12-31' ]   #指定した移植日からその年の年末まで取ってしまう。
    lalodomain = [ 36.192, 37.158, 139.327, 140.298]   #県の領域は、利用者Wikiの地理情報についての詳細ページの一覧表から知ることができます。
    Met1, tim, lat, lon = AMD.GetMetData('TMP_mea', timedomain, lalodomain, url=url)	#気象データを取得
    Pref, lat, lon = AMD.GetGeoData('pref_0900', lalodomain, url=url) # 県のIDも一覧表に掲載されています。
    valimesh = np.where(np.isnan(Met1[0,:,:] * Pref), np.nan, 1)   #県の領域、かつ、気象データがあるメッシュは１
    y,x = np.where( valimesh == 1  )
    nome = len(y)  #計算対象とするメッシュの数
    
    #計算に使用する配列の準備（日付をDOYなどで表現すればずっとシンプルになりますが、日付の形で凡例をつけさせるので、準備が少々面倒です。）
    ntim = Met1.shape[0]    #移植日から大晦日までの日数
    nlat = Met1.shape[1]	#メッシュの行数
    nlon = Met1.shape[2]	#メッシュの列数
    DVI  = np.ones(ntim)    #特定メッシュにおける日毎のDVIの変化を仮置きする配列
    dat = np.zeros((nlat, nlon)) #DVI=>100となる日(移植日からの日数)を格納する配列
    dates = np.ma.ones((nlat, nlon), dtype='datetime64[D]') #DVI=>100となる日(datetime64[D]形式)を格納する配列
    dates = np.ma.masked_where(np.isnan(valimesh), dates)   #県の領域でない、または、気象データがないメッシュにマスクを掛ける(グラフ表示しないようにする)

    #出穂日分布の計算
    for i in range(nome):
        Ta = Met1[:,y[i],x[i]]  #メッシュ[yi,xi]における、移植日から年末までの気温を取り出す
        DVI[0] = DVR100(Ta[0])	#移植日のDVIはその日のDVR
        for t in range(1,ntim):	#日付のループ
            DVI[t] = DVI[t-1] + DVR100(Ta[t])   #ｔ日目におけるDVIの積算
        tEM = np.searchsorted(DVI,100.0)        #DVIが100になる直前の日に相当するt
        if tEM == ntim: #大晦日まで出穂しない場合にエラーとならないようにする処理
            tEM = ntim - 1
        dat[y[i],x[i]] = tEM
        dates[y[i],x[i]] = np.datetime64(tim[tEM].strftime('%Y-%m-%d')) #tEM番目の日のdatetime形式オブジェクトをdatetime64[D]形式オブジェクトに変換する(そうしないと分布図が描画できない)
    dat = dat * valimesh

    #分布の描画
    tate = 6
    yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
    fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
    sclint = 1  #何日ごとに色分けするか
    sclmin = np.datetime64(year+'-07-21')		#何月何日から色を付けるか
    sclmax = np.datetime64(year+'-08-20')		#何月何日まで色を付けるか
    levels = np.arange(sclmin, sclmax+np.timedelta64(sclint,'D')+sclint, sclint)
    plt.axes(axisbg='0.8')						#背景を灰色に
    cmap = plt.cm.Spectral						#色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
    cmap.set_over('w', 1.0)						#上限を超えたときは白色
    cmap.set_under('k', 1.0)					#下限を超えたときは黒色
    CF = plt.contourf(lon, lat, dates, levels, cmap=cmap, extend='both')	#分布図を描く
    CB = plt.colorbar(CF, format=DateFormatter('%b %d'))				#カラーバーを描く
    plt.title('Date of Heading')										#図のタイトル
    #分布図画像のファイル出力
    plt.savefig('Date_of_Heading.png', dpi=600)							#図をビットマップ画像にする
    plt.show(fig)														#図を表示する
    plt.clf()
    #出穂日数分布のCSVファイル出力
    fname='Date_of_Heading-'+year+'.CSV'
    AMD.PutCSV_Map(dat,lat,lon,filename=fname)	#出穂日数の分布図をCSVでファイル出力
#    AMD.PutKMZ_Map(dat,lat,lon,label="Heading Date (Days after transplanting)", cmap="Spectral",minmax=[70,100], filename='Date_of_Heading')
