# -*- coding: utf-8 -*-
"""
AMD_Tools3の関数「GetGeoData」の利用法を理解するためのサンプルプログラム

GetGeoDataは都道府県の領域や土地利用などの理知情報をOPeNDAPサーバーまたはローカルファイルから取得する関数です。
書式：
    GetGeoData(element, lalodomain, area=None, cli=False, namuni=False, url='http://mesh.dc.affrc.go.jp/opendap')
引数(必須)：
    element：気象要素記号で、'TMP_mea'などの文字列で与える
    lalodomain：取得するデータの緯度と経度の範囲で、
        [36.0, 40.0, 130.0, 135.0] のように緯度緯度経度経度の順で指定する
        特定地点のデータを取得するときは、緯度と経度にそれぞれ同じ値を与える。
引数(必要に応じ)：
    namuni:True => 気象要素の正式名称と単位を取り出す。戻り値の数は2つ増えて6つになる。
        False => 気象要素の正式名称を取り出さない。戻り値の数は4つ(気象値、時刻、緯度、経度)。
    area:データを読み出すエリア(Area1～Area6)を指定する。省略した場合は自動的に選ばれる。
    url:データファイルの場所を指定する。省略した場合はopendapサーバーに読みに行く。
        ローカルにあるファイルを指定するときは、AreaN(N=1～6)の直上(通常は"・・・/AMD")を指定する。
戻り値：
    第1戻り値：指定した地理情報値の二次元データ。[緯度、経度]の次元を持つ。
    第2戻り値：切り出した地理情報値の緯度の並び。実数の一次元配列である。
    第3戻り値：切り出した地理情報値の経度の並び。実数の一次元配列である。
    第4戻り値(namuni=Trueのときのみ)：地理情報の正式名。文字列である。
    第5戻り値(namuni=Trueのときのみ)：地理情報の単位。文字列である。


　このサンプルプログラムでは、データ配信サーバーからデータを取り出して分布図表示させてみます
　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import AMD_Tools3 as AMD
import numpy as np
import matplotlib.pylab as plt


# 基本的な設定
element = 'landuse_H210100'                 #地理情報の指定。landuse_H210100は水田面積率を意味します。
lalodomain = [ 34.5, 36.0, 137.0, 139.5]    #領域の設定。駿河湾周辺です。

# 地理データの取得
Msh,lat,lon,nam,uni = AMD.GetGeoData(element,lalodomain,namuni=True)


#以下、分布図を書くのに重宝します。必要に応じてコピペして使用してください。－－－－－－－－
D2D = Msh       #分布図はD2Dという名の変数に入れてください。
#lat = lat      緯度はlatという名の変数に入れてください。
#lon = lon      経度はLonという名の変数に入れてください。
tate = 6      #図を入れるオブジェクト(入れ物)の全体的な大きさを指定します。
figtitle = nam+" ["+uni+"]"         
manualscl = False    #カラースケールの上限値と下限値を指定したいときに使用します。
#－－－－－－－－
sclmax  = 100.0              #最大値
sclmin  =   0.0              #最小値
sclint  =  10.0              #色の刻み
if not manualscl :
    sclmax = round(np.nanmax(np.array(D2D)),1)
    sclmin = round(np.nanmin(np.array(D2D)),1)
    sclint = round(((sclmax-sclmin)/10.0),1)
levels = np.arange(sclmin, sclmax+sclint, sclint)
yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
plt.axes(axisbg='0.8')    #背景を灰色に
levels = np.arange(sclmin, sclmax+sclint, sclint)
cmap = plt.cm.Spectral_r    #色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
cmap.set_over('w', 1.0)
cmap.set_under('k', 1.0)
CF = plt.contourf(lon, lat, D2D, levels, cmap=cmap, extend='both')
plt.colorbar(CF)
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文を有効にすると、図をpng形式で保存します。
plt.show()
plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－

# 基本的な設定
element = 'pref_2200'                 #地理情報の指定。pref_2200は静岡県域を意味します。

# 地理データの取得
Msh2,lat,lon,nam2,uni2 = AMD.GetGeoData(element,lalodomain,namuni=True)
"""
　今度は、静岡県の県域のデータを取得します。このデータには、当該県が含まれるメッシュに１、
全く含まれないメッシュに無効値(nan)が与えられています。

"""


#以下、分布図を書くのに重宝します。必要に応じてコピペして使用してください。－－－－－－－－
D2D = Msh2       #分布図はD2Dという名の変数に入れてください。
#lat = lat      緯度はlatという名の変数に入れてください。
#lon = lon      経度はLonという名の変数に入れてください。
tate = 6      #図を入れるオブジェクト(入れ物)の全体的な大きさを指定します。
figtitle = nam2+" ["+uni2+"]"         
manualscl = True    #カラースケールの上限値と下限値を指定したいときに使用します。
#－－－－－－－－
sclmax  =  1.0              #最大値
sclmin  = -1.0              #最小値
sclint  =  1.0              #色の刻み
if not manualscl :
    sclmax = round(np.nanmax(np.array(D2D)),1)
    sclmin = round(np.nanmin(np.array(D2D)),1)
    sclint = round(((sclmax-sclmin)/10.0),1)
levels = np.arange(sclmin, sclmax+sclint, sclint)
yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
plt.axes(axisbg='0.8')    #背景を灰色に
levels = np.arange(sclmin, sclmax+sclint, sclint)
cmap = plt.cm.Spectral_r    #色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
cmap.set_over('w', 1.0)
cmap.set_under('k', 1.0)
CF = plt.contourf(lon, lat, D2D, levels, cmap=cmap, extend='both')
plt.colorbar(CF)
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文を有効にすると、図をpng形式で保存します。
plt.show()
plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－

ShizPadd = Msh * Msh2
"""
　静岡県下の水田面積率布図を作成するには、水田面積率図と静岡県域のデータを掛け算します。
このように書くと、2枚の分布図の対応するメッシュすべてに対して掛け算が実行されます。無効値nan
には何を掛けてもnanになるので、結果として、静岡県域の以外が無効値となった分布図が
得られます。
"""

#以下、分布図を書くのに重宝します。必要に応じてコピペして使用してください。－－－－－－－－
D2D = ShizPadd       #分布図はD2Dという名の変数に入れてください。
#lat = lat      緯度はlatという名の変数に入れてください。
#lon = lon      経度はLonという名の変数に入れてください。
tate = 6      #図を入れるオブジェクト(入れ物)の全体的な大きさを指定します。
figtitle = nam+" ["+uni+"]"         
manualscl = True    #カラースケールの上限値と下限値を指定したいときに使用します。
#－－－－－－－－
sclmax  =  100.0              #最大値
sclmin  =    0.0              #最小値
sclint  =    5.0              #色の刻み
if not manualscl :
    sclmax = round(np.nanmax(np.array(D2D)),1)
    sclmin = round(np.nanmin(np.array(D2D)),1)
    sclint = round(((sclmax-sclmin)/10.0),1)
levels = np.arange(sclmin, sclmax+sclint, sclint)
yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
plt.axes(axisbg='0.8')    #背景を灰色に
levels = np.arange(sclmin, sclmax+sclint, sclint)
cmap = plt.cm.Spectral_r    #色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
cmap.set_over('w', 1.0)
cmap.set_under('k', 1.0)
CF = plt.contourf(lon, lat, D2D, levels, cmap=cmap, extend='both')
plt.colorbar(CF)
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文を有効にすると、図をpng形式で保存します。
plt.show()
plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－

"""
　メッシュ農業気象システムにどのような地理情報が用意されているかは、利用者Wikiに一覧が
掲載されています。下記を参照してください。
https://ml-wiki.sys.affrc.go.jp/MeshUser/geodata
　(Wikiの閲覧にはログインが必要です)
"""




