# -*- coding: utf-8 -*-
"""
AMD_Tools3の関数「GetMetData」の利用法を理解するためのサンプルプログラム

GetMetDataは気象データをOPeNDAPサーバーまたはローカルファイルから取得する関数です。
書式：
    GetMetData(element, timedomain, lalodomain, area=None, cli=False, namuni=False, url='http://mesh.dc.affrc.go.jp/opendap')
引数(必須)：
    element：気象要素記号で、'TMP_mea'などの文字列で与える
    timedomain：取得するデータの時間範囲で、['2008-05-05', '2008-05-05']
                のような文字列の2要素リストで与える。特定の日のデータを
                取得するときは、二カ所に同じ日付を与える。
    lalodomain：取得するデータの緯度と経度の範囲で、
                [36.0, 40.0, 130.0, 135.0] のように緯度経度の順で指定する
                特定地点のデータを取得するときは、緯度と経度にそれぞれ同じ
                値を与える。
引数(必要なものだけ使用)：
    cli:True => 平年値が返される。
        False => 観測値が返される。
    namuni:True => 気象要素の正式名称と単位を取り出す。戻り値の数は2つ増えて6つになる。
        False => 気象要素の正式名称を取り出さない。戻り値の数は4つ(気象値、時刻、緯度、経度)。
    area:データを読み出すエリア(Area1～Area6)を指定する。省略した場合は自動的に選ばれる。
    url:データファイルの場所を指定する。省略した場合はopendapサーバーに読みに行く。
        ローカルにあるファイルを指定するときは、AreaN(N=1～6)の直上(通常は"・・・/AMD")を指定する。
戻り値：
    第1戻り値：指定した気象要素のマスク付きの三次元データ。
                [時刻、緯度、経度]の次元を持つ。
    第2戻り値：切り出した気象データの時刻の並び。Pythonの時刻オブジェクトの
                一次元配列である。
    第3戻り値：切り出した気象データの緯度の並び。実数の一次元配列である。
    第4戻り値：切り出した気象データの経度の並び。実数の一次元配列である。
　 第5戻り値(namuni=Trueのときのみ)：気象データの正式名。文字列である。
    第6戻り値(namuni=Trueのときのみ)：気象データの単位。文字列である。


　このサンプルプログラムでは、データ配信サーバーから気象データを取得してその数値を表示させ、
データの取得方法と取り扱いの基礎を説明します。
　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import AMD_Tools3 as AMD

# 基本的な設定
element = 'TMP_mea'                         #気象要素の指定。TMP_meaは日平均気温を意味します。
timedomain = [ "2016-01-01", "2016-01-03" ] #期間の設定。
lalodomain = [ 33.32, 33.37, 131.99, 132.05]    #領域の設定。佐田岬半島の先端あたりです。

# 気象データの取得
Msh, tim, lat, lon = AMD.GetMetData(element, timedomain, lalodomain)

print()
print(Msh)
"""
Mehには指定したメッシュデータが格納されます。
[[[         nan          nan          nan          nan          nan]
  [         nan          nan          nan          nan          nan]
  [         nan   8.42308044   8.43082428          nan          nan]
  [         nan          nan   8.19772339   8.05997372          nan]
  [         nan          nan          nan   7.82792521   7.99938965]
  [         nan          nan          nan   8.28939438   7.76876163]]

 [[         nan          nan          nan          nan          nan]
  [         nan          nan          nan          nan          nan]
  [         nan  11.34343624  11.32832718          nan          nan]
  [         nan          nan  11.1041193   10.93535423          nan]
  [         nan          nan          nan  10.71115017  10.84950733]
  [         nan          nan          nan  11.16818237  10.62569714]]

 [[         nan          nan          nan          nan          nan]
  [         nan          nan          nan          nan          nan]
  [         nan  13.74090481  13.73120308          nan          nan]
  [         nan          nan  13.49593925  13.32498169          nan]
  [         nan          nan          nan  13.09068584  13.22447491]
  [         nan          nan          nan  13.52648926  12.99135494]]]
「nan」は、無効値を意味し、海上のメッシュにはデータが与えられていないのでnanと表示されます。
メッシュデータは、南から北に格納されているので、佐田岬半島は南北が反転しているように見えます。
経度の繰り返しの括弧[]、緯度の繰り返しの括弧[]、日付の繰り返しの弧[]が組み合わさって
3重になっています。
"""             

print()    #コンソールへの表示を見やすくするために1行改行します。
print(Msh[0,2,1])
"""
リストの中の特定のデータはMsh[0,2,1]のように、時間、緯度、経度の順番を指定して取り出す
ことができます。順番は0から数え始めます。
"""             

print()
print(tim)
"""
1月1日～3日の日付オブジェクトがリスト（[]で囲まれて）格納されています。
[datetime.datetime(2016, 1, 1, 0, 0) datetime.datetime(2016, 1, 2, 0, 0)
 datetime.datetime(2016, 1, 3, 0, 0)]
"""             

print()
print(lat)
"""
latには指定した範囲に入るメッシュの中心緯度が格納されます。
[ 33.32083511  33.32916641  33.33750153  33.34583282  33.35416794  33.36249924]
"""             
             
print()
print(lon)
"""
lonには指定した範囲に入るメッシュの中心経度が格納されます。
[ 131.9937439   132.0062561   132.01875305  132.03125     132.04374695]
"""             


timedomain = [ "2016-01-01", "2016-01-01" ] #期間を1日だけにします。
Msh, tim, lat, lon = AMD.GetMetData(element, timedomain, lalodomain)

print()
print(Msh)
"""
[[[        nan         nan         nan         nan         nan]
  [        nan         nan         nan         nan         nan]
  [        nan  8.42308044  8.43082428         nan         nan]
  [        nan         nan  8.19772339  8.05997372         nan]
  [        nan         nan         nan  7.82792521  7.99938965]
  [        nan         nan         nan  8.28939438  7.76876163]]]

１日分のデータなので、実質的には2次元(緯度、経度)ですが、3次元用の入れ物[[[ ]]]に入っています。
従って、「print(Msh[2,1])」とするとエラーになります。3次元用の入れ物から2次元用の入れ物に
入れなおすには、次のようにします。
Msh = Msh[0,:,:]
"""             

Msh = Msh[0,:,:]

print("2次元容器への入れ直し")
print(Msh)
print(Msh[2,1])


Msh,tim,lat,lon = AMD.GetMetData(element,timedomain,lalodomain,cli=True)

print()
print(Msh)
"""
cli=Trueを指定すると、平年値が取得できます。
"""


Msh,tim,lat,lon,nam,uni = AMD.GetMetData(element,timedomain,lalodomain,
                                         namuni=True)

print()
print(nam)
print(uni)
"""
namuni=Trueを指定すると、取得した気象要素の英語名と単位が取得できます。
"""

