# -*- coding: utf-8 -*-
"""
AMD_Tools3の関数「GetMetData」の利用法を理解するためのサンプルプログラム

GetMetDataは気象データをOPeNDAPサーバーまたはローカルファイルから取得する関数です。
書式：
    GetMetData(element, timedomain, lalodomain, area=None, cli=False, namuni=False, url='http://mesh.dc.affrc.go.jp/opendap')
引数(必須)：
    element：気象要素記号で、'TMP_mea'などの文字列で与える
    timedomain：取得するデータの時間範囲で、['2008-05-05', '2008-05-05']
                のような文字列の2要素リストで与える。特定の日のデータを
                取得するときは、二カ所に同じ日付を与える。
    lalodomain：取得するデータの緯度と経度の範囲で、
                [36.0, 40.0, 130.0, 135.0] のように緯度経度の順で指定する
                特定地点のデータを取得するときは、緯度と経度にそれぞれ同じ
                値を与える。
引数(必要なものだけ使用)：
    cli:True => 平年値が返される。
        False => 観測値が返される。
    namuni:True => 気象要素の正式名称と単位を取り出す。戻り値の数は2つ増えて6つになる。
        False => 気象要素の正式名称を取り出さない。戻り値の数は4つ(気象値、時刻、緯度、経度)。
    area:データを読み出すエリア(Area1～Area6)を指定する。省略した場合は自動的に選ばれる。
    url:データファイルの場所を指定する。省略した場合はopendapサーバーに読みに行く。
        ローカルにあるファイルを指定するときは、AreaN(N=1～6)の直上(通常は"・・・/AMD")を指定する。
戻り値：
    第1戻り値：指定した気象要素のマスク付きの三次元データ。
                [時刻、緯度、経度]の次元を持つ。
    第2戻り値：切り出した気象データの時刻の並び。Pythonの時刻オブジェクトの
                一次元配列である。
    第3戻り値：切り出した気象データの緯度の並び。実数の一次元配列である。
    第4戻り値：切り出した気象データの経度の並び。実数の一次元配列である。
    第5戻り値(namuni=Trueのときのみ)：気象データの正式名。文字列である。
    第6戻り値(namuni=Trueのときのみ)：気象データの単位。文字列である。


　このサンプルプログラムでは、データ配信サーバーからデータを取り出して分布図表示させてみます
　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import AMD_Tools3 as AMD
import numpy as np
import matplotlib.pylab as plt

# 基本的な設定
element = 'TMP_mea'                         #気象要素の指定。TMP_meaは日平均気温を意味します。
timedomain = [ "2017-02-12", "2017-02-12" ] #期間の設定。
lalodomain = [ 32.7, 34.37, 130.99, 133.05]    #領域の設定。佐田岬半島周辺が中心です。

# 気象データの取得
Msh,tim,lat,lon,nam,uni = AMD.GetMetData(element,timedomain,lalodomain,namuni=True,url="../AMD")
Msh = Msh[0,:,:]	# データの整形(3次元→2次元への変更)。


#以下、分布図を書くのに重宝します。必要に応じてコピペして使用してください。－－－－－－－－
D2D = Msh       #分布図はD2Dという名の変数に入れてください。
#lat = lat      緯度はlatという名の変数に入れてください。
#lon = lon      経度はLonという名の変数に入れてください。
tate = 6      #図を入れるオブジェクト(入れ物)の全体的な大きさを指定します。
figtitle = nam+" ["+uni+"]  on "+tim[0].strftime('%Y-%m-%d')         
manualscl = False    #カラースケールの上限値と下限値を指定したいときに使用します。
#－－－－－－－－
sclmax  = 12.8              #最大値
sclmin  =  5.4              #最小値
sclint  =  0.1              #色の刻み
if not manualscl :
    sclmax = round(np.nanmax(np.array(D2D)),1)
    sclmin = round(np.nanmin(np.array(D2D)),1)
    sclint = round(((sclmax-sclmin)/10.0),1)
levels = np.arange(sclmin, sclmax+sclint, sclint)
yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
plt.axes(axisbg='0.8')    #背景を灰色に
#plt.axes(facecolor='0.8')    #背景を灰色に  警告が表示される場合は上の行の代わりにこちらを使ってください。
levels = np.arange(sclmin, sclmax+sclint, sclint)
cmap = plt.cm.Spectral_r    #色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
cmap.set_over('w', 1.0)
cmap.set_under('k', 1.0)
CF = plt.contourf(lon, lat, D2D, levels, cmap=cmap, extend='both')
plt.colorbar(CF)
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文を有効にすると、図をpng形式で保存します。
plt.show()
plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－