# -*- coding: utf-8 -*-
"""
AMD_Tools3の関数「GetMetData」の利用法を理解するためのサンプルプログラム

GetMetDataは気象データをOPeNDAPサーバーまたはローカルファイルから取得する関数です。
書式：
    GetMetData(element, timedomain, lalodomain, area=None, cli=False, namuni=False, url='http://mesh.dc.affrc.go.jp/opendap')
引数(必須)：
    element：気象要素記号で、'TMP_mea'などの文字列で与える
    timedomain：取得するデータの時間範囲で、['2008-05-05', '2008-05-05']
                のような文字列の2要素リストで与える。特定の日のデータを
                取得するときは、二カ所に同じ日付を与える。
    lalodomain：取得するデータの緯度と経度の範囲で、
                [36.0, 40.0, 130.0, 135.0] のように緯度経度の順で指定する
                特定地点のデータを取得するときは、緯度と経度にそれぞれ同じ
                値を与える。
引数(必要なものだけ使用)：
    cli:True => 平年値が返される。
        False => 観測値が返される。
    namuni:True => 気象要素の正式名称と単位を取り出す。戻り値の数は2つ増えて6つになる。
        False => 気象要素の正式名称を取り出さない。戻り値の数は4つ(気象値、時刻、緯度、経度)。
    area:データを読み出すエリア(Area1～Area6)を指定する。省略した場合は自動的に選ばれる。
    url:データファイルの場所を指定する。省略した場合はopendapサーバーに読みに行く。
        ローカルにあるファイルを指定するときは、AreaN(N=1～6)の直上(通常は"・・・/AMD")を指定する。
戻り値：
    第1戻り値：指定した気象要素のマスク付きの三次元データ。
                [時刻、緯度、経度]の次元を持つ。
    第2戻り値：切り出した気象データの時刻の並び。Pythonの時刻オブジェクトの
                一次元配列である。
    第3戻り値：切り出した気象データの緯度の並び。実数の一次元配列である。
    第4戻り値：切り出した気象データの経度の並び。実数の一次元配列である。
    第5戻り値(namuni=Trueのときのみ)：気象データの正式名。文字列である。
    第6戻り値(namuni=Trueのときのみ)：気象データの単位。文字列である。


　このサンプルプログラムでは、データ配信サーバーから特定メッシュのデータを取り出して日別折れ線フラフを作成します。
 
　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import AMD_Tools3 as AMD
import numpy as np
from datetime import datetime
import matplotlib.pylab as plt
import matplotlib.dates as md


# 基本的な設定
element = 'TMP_mea'                         #気象要素の指定。TMP_meaは日平均気温を意味します。
timedomain = [ "2016-10-01", "2017-10-31" ] #期間の設定。
lalodomain = [ 36.0566, 36.0566, 140.125, 140.125]  #茨城県つくば市「つくば(舘野)」

# 気象データの取得
Msh,tim,lat,lon,nam,uni = AMD.GetMetData(element,timedomain,lalodomain,namuni=True)
MshN,tim,lat,lon = AMD.GetMetData(element,timedomain,lalodomain,cli=1)
Msh = Msh[:,0,0]     #入れ物の入れ替え(3次元から1次元へ)
MshN = MshN[:,0,0]   #入れ物の入れ替え(3次元から1次元へ)


# この部分を有効にすると、積算値のグラフを作成することができます。
"""
for i in range(len(tim)):
    Msh[i] = Msh[i] + Msh[i-1]
    MshN[i] = MshN[i] + MshN[i-1]
"""




#以下、基準値と比較する折れ線グラフを書くのに重宝します。必要に応じてコピペして使いまわしてください。－－－－－－－－
D1D1 = Msh       #太線で描くデータをD1D1という名の変数に入れてください。
D1D2 = MshN      #細線で描くデータをD1D2という名の変数に入れてください。
#tim = tim      日付はtimという名の変数に入れてください。
figtitle = 'N'+str(lalodomain[0])+', E'+str(lalodomain[2])          
#－－－－－－－－－－－
D1D1 = np.array(D1D1)
D1D2 = np.array(D1D2)
tim = np.array(tim)
fig = plt.figure(num=None, figsize=(12, 4))
# ・目盛の作成
ax = plt.axes()
xmajoPos = md.DayLocator(bymonthday=[1])
xmajoFmt = md.DateFormatter('%m/%d')
ax.xaxis.set_major_locator(xmajoPos)
ax.xaxis.set_major_formatter(xmajoFmt)
xminoPos = md.DayLocator()
ax.xaxis.set_minor_locator(xminoPos)
# ・データのプロット
ax.fill_between(tim,D1D1,D1D2,where=D1D1>D1D2,facecolor='orange',alpha=0.5,interpolate=True)	#細線より高い部分を橙色に塗る
ax.fill_between(tim,D1D2,D1D1,where=D1D1<D1D2,facecolor='skyblue',alpha=0.5,interpolate=True)	#細線より低い部分を水色に塗る
ax.plot(tim, D1D1, 'k')						#太線
ax.plot(tim, D1D2, 'k', linewidth=0.3)	#細線
# ・「今日」印を付ける
p = datetime.today()							      			#今日の時刻オブジェクト
today = tim == datetime(p.year,p.month,p.day,0,0,0)		#今日の配列要素番号
plt.plot(tim[today], D1D1[today], "ro")						#今日に赤点を打つ
# ・ラベル、タイトルの付加
plt.xlabel('Date')
plt.ylabel(nam+' [' + uni + ']')
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文をコメントアウトすると、図のpngファイルは作られません。
plt.show()
plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－