# -*- coding: utf-8 -*-
"""
AMD_Tools3の関数「PutCSV_MT」の利用法を理解するためのサンプルプログラム

PutCSV_MTは、3次元の配列を、基準3次メッシュコードをキーとするテーブルの形式のCSVファイルで出力する
関数です。メッシュ農業気象データのメッシュは、基準国土3次メッシュと一致しているので、あるメッシュにおけ
る値をメッシュコードを行見出しとするテーブルにすることができます。３次メッシュポリゴンデータ(３次メッシュコー
ドを属性に持つ)を持つGISを用意し、このテーブルをインポートして３次メッシュコードをキーにして連結すると、
テーブルの値をGISで表示することができます。

書式：
    PutCSV_MT(Var, lat, lon, addlalo=False, header=None, filename='result.csv', 
        removenan=True, delimiter=',')
引数(必須)：
    Var:内容を書き出す配列変数。第0次元の内容を添え字の順に記号で区切って出力する。
    lat:配列Varの各行が位置する緯度値が格納されている配列。Varの第1次元の要素数と一致していなくてはならない。
    lon:配列Varの各列が位置する経度値が格納されている配列。Varの第2次元の要素数と一致していなくてはならない。
引数(必要に応じ指定)：
    addlalo:これをTrueにすると、3次メッシュ中心点の緯度と経度が第2フィールドと第3フィールド追加挿入される。デフォルトはFalseであり挿入されない。
    header:一行目に見出しやタイトルなど何か書き出すときはここに「header='文字列'」をして指定する。
    filename：出力されるファイルの名前。デフォルト値は'result.csv'。
    delimiter:フィールドの区切り文字。デフォルト値は','。すなわち、CSVファイルとなる。
    removenan:無効値だけのレコードを削除するかを指定するキーワード。
        True=>無効値だけのレコードを削除する。水域を含む領域を出力するときに削除すると無駄なレコードが出ない。
        False=>Varに含まれるメッシュコードをすべて出力する。
 戻り値：なし。

　このサンプルプログラムでは、データ配信サーバーからデータを取り出してCSVファイルに出力し、関数の使用
法を説明します、
　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import numpy as np
import AMD_Tools3 as AMD

# 取得するデータの設定
element = 'TMP_mea'                         #気象要素の指定。TMP_meaは日平均気温を意味します。
timedomain = [ "2016-01-01", "2016-01-04" ] #期間の設定。
lalodomain = [ 33.32, 33.37, 131.99, 132.05]    #領域の設定。佐田岬半島の先端あたりです。

# 気象データの取得
Msh,tim,lat,lon = AMD.GetMetData(element,timedomain,lalodomain)

# 気象データの書き出し
AMD.PutCSV_MT(Msh, lat, lon)
"""
上の文により、下のようなCSVファイルが作成されます。1列目が基準３次メッシュコード、2列目が2016年1月1日
の気温、3列目が同年1月2日の気温、4列目が同年1月3日の気温、5列目が同年1月4日の気温です。
lalodomainで指定した領域には、30個のメッシュが含まれますが、このうち海でないメッシュは8個だけなので、
8個のメッシュだけが出力されています。
------
50320000,8.42308,11.3434,13.7409,13.0435
50320001,8.43082,11.3283,13.7312,13.0511
50320011,8.19772,11.1041,13.4959,12.8265
50320012,8.05997,10.9354,13.325,12.6688
50320022,7.82793,10.7112,13.0907,12.4455
50320023,7.99939,10.8495,13.2245,12.5943
50320032,8.28939,11.1682,13.5265,12.8809
50320033,7.76876,10.6257,12.9914,12.3725
"""

AMD.PutCSV_MT(Msh, lat, lon, addlalo=True,
              header="メッシュコード,緯度,経度,1月1日,1月2日,1月3日,1月4日",
              filename="result2.csv") #括弧の中では、自由に改行とインデントができます。
"""
　GISに読み込ませるのであれば先のようなCSVファイルで十分ですが、メッシュ中心点の緯度経度や見出しも
 付加しておくと、あとから確認ができて便利です。上の文のようにオプション引数を付加すると、下のようなCS
 Vファイルを作成することができます。
------
メッシュコード,緯度,経度,1月1日,1月2日,1月3日,1月4日
50320000,33.3375,132.006,8.42308,11.3434,13.7409,13.0435
50320001,33.3375,132.019,8.43082,11.3283,13.7312,13.0511
50320011,33.3458,132.019,8.19772,11.1041,13.4959,12.8265
50320012,33.3458,132.031,8.05997,10.9354,13.325,12.6688
50320022,33.3542,132.031,7.82793,10.7112,13.0907,12.4455
50320023,33.3542,132.044,7.99939,10.8495,13.2245,12.5943
50320032,33.3625,132.031,8.28939,11.1682,13.5265,12.8809
50320033,33.3625,132.044,7.76876,10.6257,12.9914,12.3725
"""

timedomain = [ "2016-01-01", "2016-01-01" ] #期間の設定。
MshA,tim,lat,lon = AMD.GetMetData('TMP_mea',timedomain,lalodomain)
MshA = MshA[0,:,:]
MshB,tim,lat,lon = AMD.GetMetData('TMP_max',timedomain,lalodomain)
MshB = MshB[0,:,:]
MshC,tim,lat,lon = AMD.GetMetData('TMP_min',timedomain,lalodomain)
MshC = MshC[0,:,:]
Msh2 = np.array([MshA,MshB,MshC])
AMD.PutCSV_MT(Msh2, lat, lon,
              header="メッシュコード,Tmean,Tmax,Tmin", filename="result3.csv")
"""
　ばらばらに作成した配列を連結して出力するには、上のように、numpyモジュールに含まれるarra関数を
用います。
------
メッシュコード,Tmean,Tmax,Tmin
50320000,8.42308,12.9888,5.07289
50320001,8.43082,12.9816,5.06517
50320011,8.19772,12.5856,4.82417
50320012,8.05997,12.4761,4.73078
50320022,7.82793,12.1788,4.44593
50320023,7.99939,12.2195,4.71737
50320032,8.28939,12.6806,5.01522
50320033,7.76876,12.1676,4.4425
"""
