# -*- coding: utf-8 -*-

"""
　AMD_Tools3の関数「PutGSI_Map」の利用法を理解するためのサンプルプログラム
PutKMZ_Mapは、2次元(空間分布)の配列をhtmlファイルで出力する関数です。
この関数で作られたファイルをダブルクリックすると、メッシュ農業気象データやそれを使って作成した分布図が
国土地理院の地図サービス「地理院地図」h上で表示できます。
書式：
    PutGSI_Map(data, lat, lon, label=None, cmap=None, minmax=None, filename="result", outdir="."):
引数(必須)：
    data：表示させるデータ（2次元numpyアレイ）
    lat：緯度（1次元numpyアレイ）
    lon：経度（1次元numpyアレイ）
引数(必要に応じ指定)：
    label：凡例のタイトルの文字列
    cmap：カラーマップを指定（詳細は後述）
    minmax：カラースケールを指定（[min,max]のリスト型）
    filename：出力ファイル名
    outdir:出力フォルダ名
カラーマップについて：
    カラーマップには名称があるのでこれを文字列で("で囲んで)指定する。
    例)　レインボーカラー:rainbow、黄色-オレンジ-赤の順で変化:YlOrRdなど
    色の順序をを反転させたい場合は、rainbow_rのよう名称の後ろに"_r"を付加する。
    詳細は下記URLを参照。
        http://matplotlib.org/examples/color/colormaps_reference.html
使用例：
北緯36～38.5度、東経137.5～141.5度の範囲における2016年1月1日の平均気温分布図のKMZファイルを作成する。
import AMD_Tools3 as AMD
element = 'TMP_mea'
timedomain = [ "2016-01-01", "2016-01-01" ]
lalodomain = [ 36.0, 38.5, 137.5, 141.5]
Msh,tim,lat,lon,nam,uni = AMD.GetMetData(element, timedomain, lalodomain,namuni=True)
dat = Msh[0,:,:]
AMD.PutKMZ_Map(dat,lat,lon,label=nam+" ["+uni+"]", cmap="rainbow",minmax=None, filename=element)

　このサンプルプログラムでは、佐田岬半島周辺における2016年1月1日の日平均気温分布図を地理院地図上で表示させる
ファイルを作成します。

　このサンプルプログラムを実行するには、Python統合開発環境Spyderのエディタペインにプログラムファイル
 を読み込んでから実行ボタン(三角ボタン）をクリックしてください。

Hiroyuki OHNO, 2017.02.08
"""
import AMD_Tools3 as AMD

# 基本的な設定
element = 'TMP_mea'                         #気象要素の指定。TMP_meaは日平均気温を意味します。
timedomain = [ "2016-01-01", "2016-01-01" ] #期間の設定。
lalodomain = [ 32.7, 34.37, 130.99, 133.05]    #領域の設定。佐田岬半島周辺が中心です。

# 気象データの読み込み
Msh,tim,lat,lon,nam,uni = AMD.GetMetData(element, timedomain, lalodomain,namuni=True)
#　Mshには気象データが格納されます。
#　timには、timedomainで指定した2016年1月1日の日付オブジェクトが格納されます。
#　latには、北緯36.0～38.5度の範囲に含まれるメッシュ中心点の緯度値が格納されます。
#　lonには、東経137.5～141.5度の範囲に含まれるメッシュ中心点の経度値が格納されます。
#　namには、日平均気温が英語で格納されます。
#　uniには、気温の単位が格納されます。

# データの整形(3次元→2次元への変更)。
Msh = Msh[0,:,:]
# メッシュ農業気象データーは、日付、緯度、経度の３つの次元を持ちますが、
# 今回は1日だけのデータなので、上のようにして2次元のデータ変換して後の処理を行います。

# GoogleEarth表示用ファイル出力
AMD.PutGSI_Map(Msh,lat,lon,label=nam+" ["+uni+"]",filename=element)
#書式：
#    PutKMZ_Map(data, lat, lon, label=None, cmap=None, minmax=None, filename="result", outdir=".")
#引数(必須)：
#    data：表示させるデータ（2次元numpyアレイ）
#    lat：緯度（1次元numpyアレイ）
#    lon：経度（1次元numpyアレイ）
#引数(なくても何とかなる)：
#    label：凡例のタイトルの文字列
#    cmap：カラーマップを指定
#    minmax：カラースケールを指定（[min,max]のリスト型）
#    filename：出力ファイル名
#    outdir:出力フォルダ名
