# -*- coding: utf-8 -*-

"""
　水稲を指定した日に県下全域に移植したとした時の出穂日(移植後日数)の分布予測図を作成する
プログラム。
　気温と発育速度(DVR)との関係式を、関数としてこのプログラムで定義し用いる。
 対象地域にわたって発育を予測して作図をする本体部分は、DVR関数の定義の下から始まる。

	OHNO, Hiroyuki 2017.05.12
"""

import numpy as np
import matplotlib.pylab as plt
import AMD_Tools3 as AMD

#発育速度関数の定義部分
def DVR(Ta, Para=[10.0, 1000.0]): #関数はdef文で定義します。
    """
    積算温度型の発育速度関数。
    引数(必須)：
      Ta：日平均気温
    引数(必要に応じ設定)  
      Para:基準温度(これ以下の場合積算をしない)、ならびに、出穂到達積算温度を束ねたリスト
           デフォルト値として[10, 1000]が設定されている。
    戻り値：気温に対する発育速度値
    """
    if Para[0] < Ta:
        DVR = (Ta-Para[0])/Para[1]
    else:
        DVR = 0.0
    return DVR 

#プログラムのメイン部分
if __name__ == "__main__":
    #移植条件の設定
    year = '2017'
    Day_of_Transplanting = '05-10'
    DVIini = 0.0
    koshi = [10,950]   #DVR関数の品種パラメータ(サンプルです！)

    #メッシュデータの取得
    prefecture = 'pref_1500'	#新潟県
    lalodomain = [36.733, 38.558, 137.627, 139.898]   #県の領域は、利用者Wikiの地理情報についての詳細ページの一覧表から知ることができます。
    timedomain = [year+'-'+Day_of_Transplanting, year+'-12-31' ]   #指定した移植日からその年の年末まで取ってしまう。
    Msh, tim, lat, lon = AMD.GetMetData('TMP_mea', timedomain, lalodomain)	#気象データを取得
    Pref, lat, lon = AMD.GetGeoData(prefecture, lalodomain) # 県のID　これも一覧表に掲載されています。

    #計算すべきメッシュの選別
    valimesh = np.where(np.isnan(Msh[0,:,:] * Pref ), np.nan, 1)   #県の領域、かつ、気象データがあるメッシュは１
    y,x = np.where( valimesh == 1  )
    nome = len(y)  #計算対象とするメッシュの数
    
    #計算に使用する配列の準備
    ntim = Msh.shape[0]    #移植日から大晦日までの日数
    nlat = Msh.shape[1]	#メッシュの行数
    nlon = Msh.shape[2]	#メッシュの列数
    dat = np.zeros((nlat, nlon)) #DVI>1となる日(移植日からの日数)を格納する配列
    dat = dat * valimesh

    #出穂日分布の計算
    for i in range(nome):
        Ta = Msh[:,y[i],x[i]]  #メッシュ[yi,xi]における、移植日から年末までの気温を取り出す
        DVI = DVIini
        dem = ntim - 1
        for d in range(1,ntim):	#日付のループ
            DVI = DVI + DVR(Ta[d],Para=koshi)   #ｔ日目におけるDVIの積算
            if DVI > 1.0 : 
                dem = d        #DVIが1を超えた日に相当するt
                break
        dat[y[i],x[i]] = dem

    #以下、分布図を書くのに重宝します。必要に応じてコピペして使用してください。－－－－－－－－
    D2D = dat       #分布図はD2Dという名の変数に入れてください。
    #lat = lat      緯度はlatという名の変数に入れてください。
    #lon = lon      経度はLonという名の変数に入れてください。
    tate = 6      #図を入れるオブジェクト(入れ物)の全体的な大きさを指定します。
    figtitle ="Number of Days after Trnsplantig"
    manualscl = True    #カラースケールの上限値と下限値を指定したいときに使用します。
    #－－－－－－－－
    sclmax  = 120.0              #最大値
    sclmin  =  80.0              #最小値
    sclint  =   1.0              #色の刻み
    if not manualscl :
        sclmax = round(np.nanmax(np.array(D2D)),1)
        sclmin = round(np.nanmin(np.array(D2D)),1)
        sclint = round(((sclmax-sclmin)/10.0),1)
    levels = np.arange(sclmin, sclmax+sclint, sclint)
    yoko = tate * (np.max(lon)-np.min(lon))/(np.max(lat)-np.min(lat)) + 2
    fig = plt.figure(num=None, figsize=(yoko, tate))#図を入れるオブジェクト(入れ物)を定義
    plt.axes(axisbg='0.8')    #背景を灰色に
    #plt.axes(facecolor='0.8')    #背景を灰色に  警告が表示される場合は上の行の代わりにこちらを使ってください。
    levels = np.arange(sclmin, sclmax+sclint, sclint)
    cmap = plt.cm.Spectral    #色調(カラーマップ)を愛称(「_r」を最後に付けると反転する)で指定
    cmap.set_over('w', 1.0)
    cmap.set_under('k', 1.0)
    CF = plt.contourf(lon, lat, D2D, levels, cmap=cmap, extend='both')
    plt.colorbar(CF)
    plt.title(figtitle)
    plt.savefig('result'+'.png', dpi=600) #この文を有効にすると、図をpng形式で保存します。
    plt.show()
    plt.clf()
#－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－－
